//
//  SXEditManager.h
//  SXVideoEnging
//
//  Created by Yin Xie on 2020/3/31.
//  Copyright © 2020 Zhiqiang Li. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "SXEditOptions.h"
#import "SXTrack.h"
#import "SXTrackGroup.h"
#import "SXAudioItem.h"
NS_ASSUME_NONNULL_BEGIN


/**
 * SXEditManager委托
 */
@protocol SXEditManagerDelegate <NSObject>

/**
 * 播放进度更新
 * @param frameIndex 当前播放的帧序列号
 */
- (void)playFrameIndex:(NSInteger)frameIndex;

/**
 * 渲染开始
 */
-(void)renderStarted;

/**
 * 渲染结束
 * @param tempUrl 结果文件路径
 */
-(void)renderFinished:(NSURL *)tempUrl;

/**
 * 渲染被取消
 */
-(void)renderCancelled;

/**
 渲染失败

 @param error 错误信息
 @warning 当erro中userinfo包含request， 且为"/render/charge/cpt/confirm", 请记录路径和订单流水号(必须对应)，以便稍后调用decodeFile重新尝试输出视频文件
          渲染回调的文件如果不是直接再次导出，请注意保存该文件，不然可能会被覆盖
 */
-(void)renderFailed:(NSError *)error;
/**
 * 渲染进度更新
 * @param progress 渲染进度，0~1
 */
-(void)renderProgress:(float) progress;

@end


@interface SXEditManager : NSObject

/**
 * SXEditManager委托
 */
@property (nonatomic, weak) id<SXEditManagerDelegate> delegate;

@property (nonatomic, strong, readonly) SXEditOptions *options;

/**
 * 初始化编辑管理类
 * @param options 初始化参数，初始化时生效，后期修改无效
*/
- (instancetype)initWithOptions:(SXEditOptions *)options;

/**
 * SDK内部使用帧数计算，消除时间转化中的误差
 * @return 去掉误差后的时间
*/
- (double)removeTimeError:(double)time;

/**
 * 获取主轨道组，主轨道是一个不存在间隔的组。
 * @note 默认创建且不允许删除
 * @warning 需要license支持才能返回其指针，否则返回nullptr
 * @note 主轨道仅接受插入Media类型的轨道
 * @return 主轨道实例对象，license不支持时返回nullptr
 */
- (SXMainTrackGroup *)mainGroup;

/**
 * 通过id获取对应的组，id可通过类方法uuid获取
 * @param groupId 组的uuid
 * @return 组的实例对象，不存在对应的组时返回nullptr
 */
- (SXTrackGroup *)group:(NSString *)groupId;

/**
 * 在最后方追加一个新组
 * @return 新组的实例对象
 */
- (SXTrackGroup *)addNewGroup;

/**
 * 组是顺序存储的，因此可以在index所在的位置插入一个组
 * @param index 位置的索引，index超出组个数时等价于addNewGroup
 * @return 新组的实例对象
 */
- (SXTrackGroup *)insertNewGroupAt:(int)index;

/**
 * 通过id删除组，如果组内存在轨道则会移除所有轨道，但不会删除轨道
 * @param groupId 组的唯一id
 */
- (void)removeGroup:(NSString *)groupId;

/**
 * 移动一个组到指定的位置
 * @param groupId 组的唯一id
 * @param index 组的顺序索引，从0开始（不包含主轨道和当前轨道）
 * @return 组查找失败或索引越界返回false
 */
- (BOOL)removeGroup:(NSString *)groupId toIndex:(int)index;

/**
 * 获取所有的组
 * @return 顺序存储的所有组
 */
- (NSArray *)groups;

/**
 * 创建媒体轨道，接受视频和图片素材
 * @param sourcePath 资源路径
 * @return 媒体轨道
 */
- (SXMediaTrack *)createMediaTrack:(NSString *)sourcePath;

/**
 * 创建贴纸轨道
 * @param sourcePath 资源路径
 * @param duration  持续时长
 * @return 贴纸轨道
 */
- (SXStickerTrack *)createStickerTrack:(NSString *)sourcePath duration:(double)duration;

/**
 * 创建文字轨道
 * @param duration 持续时长
 * @return 文字轨道
 * */
- (SXTextTrack *)createTextTrack:(double)duration;

/**
 * 通过轨道id删除一个轨道， 该track将被释放，后续不能再使用
 * @param trackId 轨道id
 * @param removeFromGroup 是否需要遍历组以从组中移除此轨道（建议在删除前先从组中移除轨道）
 */
- (void)deleteTrack:(NSString *)trackId removeFromGroupbool:(BOOL)removeFromGroup;

/**
 * 通过轨道id复制一个新的轨道
 * @param trackId 轨道id
 * @return    新的轨道，复制失败时返回空指针
 */
- (SXTrack *)cloneTrack:(NSString *)trackId;

/**
 * 创建音频轨道
 * @param audioPath 音频路径
 * @param inPoint 音频插入的时间点
 * @return 音频轨道
 * */
- (SXAudioItem *)createAudioItem:(NSString *)audioPath inPoint:(double)inPoint;

/**
 * 获取音频轨道
 * @param audioId 音频Id
*/
- (SXAudioItem *)getAudioItemWithId:(NSString *)audioId;

/**
 * 通过音频id删除一个音频， 该音频将被释放，后续不能再使用
 * @param audioId 音频id
*/
- (void)deleteAudioItem:(NSString *)audioId;

/**
 * 通过音频id复制一个新的音频轨道
 * @param audioId 音频id
 * @return    新的轨道，复制失败时返回空指针
 */
- (SXAudioItem *)cloneAudioItem:(NSString *)audioId;

/**
 * in point   track     split time     new track    out point
 *   |-----------------------|--------------------------|
 * ----------------------------------------------------------------------
 * 拆分音频轨道
 * @note 拆分后的新轨道会自动加入音频组
 * @param audioId 音频id
 * @param time 相对于轨道自身in point拆分时间点
 * @return 查找轨道失败或拆分时间不在轨道显示范围内返回nullptr，否则返回轨道实例对象
 */
- (SXAudioItem *)splitAudioItem:(NSString *)audioId time:(double)time;

/**
 * 获取所有音频轨道
 * @return 音频轨道数组
*/
- (NSArray *)getAudioItems;

/**
 * 获取当前总时长
 *
 * @return 总时长: 秒
 */

- (double)editDuration;

/**
 * 设置背景颜色, 默认黑色
 * @param red [0-1]
 * @param green [0-1]
 * @param blue [0-1]
 * */
- (void)setBackgroundColorRed:(CGFloat)red green:(CGFloat)green blue:(CGFloat)blue;

/**
 * 设置背景视频或图片
 *
 * @param resourcePath 背景资源
 * @return error code
 */
- (int)setBackground:(NSString *)resourcePath;

/**
 * 删除背景视频或图片
 */
- (void)removeBackground;

/**
 * 根据point查找画布中的track
 *
 * @param point 相对于画布左上角点的坐标
 *
 * @return 如果有相应的track返回对象，否则返回nil
*/
- (SXTrack *)hitPoint:(CGPoint)point;

/**
 * 根据point查找画布中相应类型的track
 *
 * @param point 相对于画布左上角点的坐标
 * @param type 需要查找的track类型
 *
 * @return 如果有相应的track返回对象，否则返回nil
*/
- (SXTrack *)hitPoint:(CGPoint)point type:(SXTrackType)type;

/**
 * 重设 edit 尺寸
 *
 * @param size 尺寸
 * @return 是否成功
 */
- (BOOL)resetEditSize:(CGSize)size;


#pragma mark - Play and Render
/**
 * 开始预览
 * @return 是否成功
 */
- (BOOL)start;

/**
 * 暂停预览
 * @return 是否成功
 */
- (BOOL)pause;

/**
 * 停止预览,释放资源
 * @return 是否成功
 */
- (BOOL)stop;

/**
 * 获取播放状态
 * @return 是否正在播放
 */
- (BOOL)isPlay;

/**
 * 开始渲染
 * @return 是否开始成功
 */
- (BOOL)startRender;

/**
 * 取消渲染
 */
- (void)cancelRender;

/**
 * 设置时间进度
 * @param second 新的时间进度，秒为单位
 * @return 是否设置成功
 */
- (BOOL)seek:(float)second;

/**
 * 获取播放器所使用的视图对象
 * @return UIView视图对象，获取到之后可以添加显示到界面上
 */
- (UIView *)getPlayerView;

/**
 录制文件导出失败后重试接口,参数由录制回调的错误中获得
 
 @param filePath 加密文件路径, 对应key - filePath
 @param orderSN 订单流水号, 对应key - orderSN
 @param featureString 订单功能, 对应key - featureString
 @param outputPath 解密文件输出路径
 @param completeHandler 完成回调
 
 @warning 一个订单对应一个加密文件，参数不要修改，不要用来解密其他文件，可能造成文件错误且扣费成功
 解压完成后不会删除原文件
 */
+ (void)decodeFile:(NSString *)filePath
           orderSN:(NSString *)orderSN
     featureString:(NSString *)featureString
        outputPath:(NSString *)outputPath
   completeHandler:(void (^)(NSError *))completeHandler;

@end

NS_ASSUME_NONNULL_END
